package com.ruoyi.cstkb.service.impl;

import com.ruoyi.cstkb.domain.CstObjectInfo;
import com.ruoyi.cstkb.mapper.CstObjectInfoMapper;
import com.ruoyi.cstkb.mapper.CstPropertiesInfoMapper;
import com.ruoyi.cstkb.service.ICstObjectInfoService;
import com.ruoyi.cstkb.web.request.CstObjNodeRequest;
import com.ruoyi.cstkb.web.request.CstPropertiesRequest;
import com.ruoyi.cstkb.web.response.CstObjNodeResponse;
import com.ruoyi.cstkb.web.response.CstPropertiesResponse;
import org.apache.commons.lang3.StringUtils;
import org.json.JSONException;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 汽车安全测试对象信息Service业务层处理
 *
 * @author ruoyi
 * @date 2024-10-12
 */
@Service
public class CstObjectInfoServiceImpl implements ICstObjectInfoService {
    @Autowired
    private CstObjectInfoMapper cstObjectInfoMapper;

    @Autowired
    private CstPropertiesInfoMapper cstPropertiesInfoMapper;

    /**
     * 查询汽车安全测试对象信息
     *
     * @param id 汽车安全测试对象信息主键
     * @return 汽车安全测试对象信息
     */
    @Override
    public CstObjectInfo selectCstObjectInfoById(Long id) {
        return cstObjectInfoMapper.selectCstObjectInfoById(id);
    }

    /**
     * 查询汽车安全测试对象信息列表
     *
     * @param cstObjectInfo 汽车安全测试对象信息
     * @return 汽车安全测试对象信息
     */
    @Override
    public List<CstObjectInfo> selectCstObjectInfoList(CstObjectInfo cstObjectInfo) {
        return cstObjectInfoMapper.selectCstObjectInfoList(cstObjectInfo);
    }

    /**
     * 新增汽车安全测试对象信息
     *
     * @param cstObjectInfo 汽车安全测试对象信息
     * @return 结果
     */
    @Override
    public int insertCstObjectInfo(CstObjectInfo cstObjectInfo) {
        return cstObjectInfoMapper.insertCstObjectInfo(cstObjectInfo);
    }

    /**
     * 修改汽车安全测试对象信息
     *
     * @param cstObjectInfo 汽车安全测试对象信息
     * @return 结果
     */
    @Override
    public int updateCstObjectInfo(CstObjectInfo cstObjectInfo) {
        return cstObjectInfoMapper.updateCstObjectInfo(cstObjectInfo);
    }

    /**
     * 批量删除汽车安全测试对象信息
     *
     * @param ids 需要删除的汽车安全测试对象信息主键
     * @return 结果
     */
    @Override
    public int deleteCstObjectInfoByIds(Long[] ids) {
        return cstObjectInfoMapper.deleteCstObjectInfoByIds(ids);
    }

    /**
     * 删除汽车安全测试对象信息信息
     *
     * @param id 汽车安全测试对象信息主键
     * @return 结果
     */
    @Override
    public int deleteCstObjectInfoById(Long id) {
        return cstObjectInfoMapper.deleteCstObjectInfoById(id);
    }

    /**
     * 节点关系图
     * 条件查询 关系图节点对象 数据列表
     *
     * @param cstObjNodeRequest
     * @return
     */
    @Override
    public List<CstObjNodeResponse> getObjNodeList(CstObjNodeRequest cstObjNodeRequest) {
        List<CstObjNodeResponse> relationNodeList = cstObjectInfoMapper.getObjNodeList(cstObjNodeRequest);
        // 获取 全部场景对象id
        CstPropertiesRequest prosQueryParams = new CstPropertiesRequest();
        List<String> scenarioObjIds = relationNodeList.stream()
                .map(CstObjNodeResponse::getScenarioObjId)
                .collect(Collectors.toList());
        prosQueryParams.setScenarioObjIds(scenarioObjIds);
        // 根据 全部场景对象id 查询全部属性
        List<CstPropertiesResponse> allPropertiesList = cstPropertiesInfoMapper.getObjNodeProsList(prosQueryParams);
        // 根据 全部场景对象id 进行分组
        Map<String, List<CstPropertiesResponse>> propertiesMap = allPropertiesList.stream()
                .filter(s -> StringUtils.isNotEmpty(s.getScenarioObjId()))
                .collect(Collectors.groupingBy(CstPropertiesResponse::getScenarioObjId));
        // 根据 全部场景对象id 设置所含全部属性
        relationNodeList.stream()
                .filter(itemNode -> propertiesMap.containsKey(itemNode.getScenarioObjId()))
                .forEach(itemNode -> itemNode.setPropertiesInfoList(propertiesMap.get(itemNode.getScenarioObjId())));
        return relationNodeList;
    }

    /**
     * 获取 汽车安全测试知识库  首页圆形气泡图 Json 数据
     *
     * @return
     */
    public JSONObject getIndexGraphJsonData() throws JSONException {
        // json数据单元
        JSONObject cellJson = new JSONObject();
        cellJson.put("$count", 1);
        // 需要返回的 Json 数据
        JSONObject graphJson = new JSONObject();
        CstObjNodeRequest queryParams = new CstObjNodeRequest();
        List<CstObjNodeResponse> objNodeList = cstObjectInfoMapper.getObjNodeList(queryParams);
        // 按照 子场景id 对 对象节点 list 进行分组
        Map<Long, List<CstObjNodeResponse>> subMap = objNodeList.stream()
                .filter(ssid -> ssid.getScenarioSubclassId() != null)
                .collect(Collectors.groupingBy(CstObjNodeResponse::getScenarioSubclassId,
                        TreeMap::new,
                        Collectors.toList()));
        // 按照 主场景id 对 对象节点 list 进行分组
        Map<Long, List<CstObjNodeResponse>> mainMap = objNodeList.stream()
                .filter(msid -> msid.getScenarioMainclassId() != null)
                .collect(Collectors.groupingBy(CstObjNodeResponse::getScenarioMainclassId,
                        TreeMap::new,
                        Collectors.toList()));
        // 组装 Json 数据
        subMap.forEach((key, value) -> {
            JSONObject subJson = new JSONObject();
            for (CstObjNodeResponse tempObj : value) {
                JSONObject objJson = new JSONObject();
                try {
                    objJson.put(tempObj.getObjName(), cellJson);
                    subJson.put("$count", objJson.length());
                    if (subJson.has(value.get(0).getScenarioSubclassName())) {
                        // 当 ScenarioSubclassName 存在时为添加 objJson
                    } else {
                        subJson.put(value.get(0).getScenarioSubclassName(), objJson);
                    }
                } catch (JSONException e) {
                    throw new RuntimeException(e);
                }
            }

        });
        return graphJson;
    }


    /**
     * 获取 汽车安全测试知识库 首页圆形气泡图 Json 数据
     *
     * @return
     */
    public JSONObject getIndexGraphJsonData2() throws JSONException {
        // 需要返回的 Json 数据
        JSONObject graphJson = new JSONObject();
        CstObjNodeRequest queryParams = new CstObjNodeRequest();
        List<CstObjNodeResponse> objNodeList = cstObjectInfoMapper.getObjNodeList(queryParams);

        // 创建层级结构
        Map<String, Map<String, Map<String, List<Long>>>> hierarchy = new HashMap<>();

        for (CstObjNodeResponse obj : objNodeList) {
            // 主场景 ID
            Long mainLayerId = obj.getScenarioMainclassId();
            // 主场景名
            String mainLayer = obj.getScenarioMainclassName();
            // 子场景 ID
            Long subLayerId = obj.getScenarioSubclassId();
            // 子场景名
            String subLayer = obj.getScenarioSubclassName();
            // 对象 ID
            Long objId = obj.getId();
            // 对象名
            String objName = obj.getObjName();

            // 初始化层级结构
            hierarchy.putIfAbsent(mainLayer, new HashMap<>());
            hierarchy.get(mainLayer).putIfAbsent(subLayer, new HashMap<>());

            // 将对象 ID 添加到列表中
            hierarchy.get(mainLayer).get(subLayer).putIfAbsent(objName, new ArrayList<>());
            hierarchy.get(mainLayer).get(subLayer).get(objName).add(objId);
        }

        // 组装最终的 JSON 数据
        for (Map.Entry<String, Map<String, Map<String, List<Long>>>> mainEntry : hierarchy.entrySet()) {
            JSONObject mainLayerJson = new JSONObject();
            mainLayerJson.put("$count", mainEntry.getValue().size()); // 主层级计数

            for (Map.Entry<String, Map<String, List<Long>>> subEntry : mainEntry.getValue().entrySet()) {
                JSONObject subLayerJson = new JSONObject();
                subLayerJson.put("$count", subEntry.getValue().size()); // 子层级计数

                for (Map.Entry<String, List<Long>> objEntry : subEntry.getValue().entrySet()) {
                    String name = objEntry.getKey(); // 对象名称

                    // 将对象名称及其 ID 列表作为子节点
                    JSONObject objJson = new JSONObject();
                    objJson.put("$count", objEntry.getValue().size()); // 对象计数
                    objJson.put("ids", objEntry.getValue()); // 存储对象 ID 列表
                    subLayerJson.put(name, objJson);
                }

                mainLayerJson.put(subEntry.getKey(), subLayerJson);
            }

            graphJson.put(mainEntry.getKey(), mainLayerJson);
        }

        return graphJson;
    }

    public JSONObject getIndexGraphJsonData3() throws JSONException {
        // json数据单元
        JSONObject cellJson = new JSONObject();
        cellJson.put("$count", 1);
        // 需要返回的 Json 数据
        JSONObject graphJson = new JSONObject();
        CstObjNodeRequest queryParams = new CstObjNodeRequest();
        List<CstObjNodeResponse> objNodeList = cstObjectInfoMapper.getObjNodeList(queryParams);
        // 按照 子场景id 对 对象节点 list 进行分组
        Map<Long, List<CstObjNodeResponse>> subMap = objNodeList.stream()
                .filter(ssid -> ssid.getScenarioSubclassId() != null)
                .collect(Collectors.groupingBy(CstObjNodeResponse::getScenarioSubclassId,
                        TreeMap::new,
                        Collectors.toList()));
        // 按照 主场景id 对 对象节点 list 进行分组
        Map<Long, List<CstObjNodeResponse>> mainMap = objNodeList.stream()
                .filter(msid -> msid.getScenarioMainclassId() != null)
                .collect(Collectors.groupingBy(CstObjNodeResponse::getScenarioMainclassId,
                        TreeMap::new,
                        Collectors.toList()));
        // 组装 Json 数据
        subMap.forEach((key, value) -> {
            JSONObject subJson = new JSONObject();
            try {
                subJson.put("$count", value.size());
                subJson.put("id", value.get(0).getScenarioSubclassId());
            } catch (JSONException e) {
                throw new RuntimeException(e);
            }
            JSONObject objJson = new JSONObject();
            for (CstObjNodeResponse tempObj : value) {
                try {
                    objJson.put(tempObj.getObjName(), cellJson);
                    if (!subJson.has(value.get(0).getScenarioSubclassName())) {
                        // 当 ScenarioSubclassName 不存在时为添加 objJson
                        subJson.put(value.get(0).getScenarioSubclassName(), null);
                    }
                } catch (JSONException e) {
                    throw new RuntimeException(e);
                }
            }
            if (!graphJson.has(value.get(0).getScenarioMainclassName())) {
                try {
                    graphJson.put("id", value.get(0).getScenarioMainclassId());
                    graphJson.put("$count", value.size());
                    graphJson.put(value.get(0).getScenarioMainclassName(), subJson);
                } catch (JSONException e) {
                    throw new RuntimeException(e);
                }
            } else {
            }
        });
        return graphJson;
    }
}
